@@ -2,7 +2,7 @@
 // detail/win_thread.hpp
 // ~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2019 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -21,6 +21,7 @@
   && !defined(ASIO_WINDOWS_APP) \
   && !defined(UNDER_CE)
 
+#include <cstddef>
 #include "asio/detail/noncopyable.hpp"
 #include "asio/detail/socket_types.hpp"
 
@@ -78,6 +79,9 @@ public:
   // Wait for the thread to exit.
   ASIO_DECL void join();
 
+  // Get number of CPUs.
+  ASIO_DECL static std::size_t hardware_concurrency();
+
 private:
   friend ASIO_DECL unsigned int __stdcall win_thread_function(void* arg);
 
