@@ -1,4 +1,4 @@
-/* Copyright (C) 2011-2016 Codership Oy <info@codership.com> */
+/* Copyright (C) 2011-2023 Codership Oy <info@codership.com> */
 
 #ifndef _GARB_RECV_LOOP_HPP_
 #define _GARB_RECV_LOOP_HPP_
@@ -25,7 +25,9 @@ public:
 
 private:
 
+    bool one_loop();
     void loop();
+    void close_connection();
 
     const Config& config_;
     gu::Config    gconf_;
@@ -35,10 +37,7 @@ private:
         RegisterParams(gu::Config& cnf)
         {
             gu::ssl_register_params(cnf);
-            if (gcs_register_params(reinterpret_cast<gu_config_t*>(&cnf)))
-            {
-                gu_throw_fatal << "Error initializing GCS parameters";
-            }
+            gcs_register_params(cnf);
             cnf.add(COMMON_BASE_DIR_KEY);
         }
     }
@@ -59,6 +58,7 @@ private:
     gu::UUID    uuid_;
     gu::seqno_t seqno_;
     int         proto_;
+    bool        closed_;
 
 }; /* RecvLoop */
 
