@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2025 Codership Oy <info@codership.com>
  */
 
 /*! @file page file class implementation */
@@ -46,15 +46,21 @@ gcache::Page::drop_fs_cache() const
 #endif
 }
 
-gcache::Page::Page (void* ps, const std::string& name, size_t size, int dbg)
+gcache::Page::Page (void* const        ps,
+                    const std::string& name,
+                    size_t             size,
+                    int                dbg)
     :
     fd_   (name, size, true, false),
     mmap_ (fd_),
+    seqno_max_(SEQNO_NONE),
     ps_   (ps),
     next_ (static_cast<uint8_t*>(mmap_.ptr)),
     space_(mmap_.size),
-    used_ (0),
-    debug_(dbg)
+    used_(0),
+    mapped_(0),
+    debug_(dbg),
+    closed_(false)
 {
     log_info << "Created page " << name << " of size " << space_
              << " bytes";
@@ -66,7 +72,7 @@ gcache::Page::malloc (size_type size)
 {
     Limits::assert_size(size);
 
-    if (size <= space_)
+    if (size <= space_ && !closed_)
     {
         BufferHeader* bh(BH_cast(next_));
 
@@ -90,7 +96,8 @@ gcache::Page::malloc (size_type size)
 
         assert (next_ <= static_cast<uint8_t*>(mmap_.ptr) + mmap_.size);
 
-        if (debug_) { log_info << name() << " allocd " << bh; }
+        if (debug_) { log_info << name() << " allocd " << bh << ", used: "
+                               << used_ << ", mapped: " << mapped_; }
 #endif
 
         return (bh + 1);
@@ -108,6 +115,7 @@ void*
 gcache::Page::realloc (void* ptr, size_type size)
 {
     Limits::assert_size(size);
+    if (closed_) return nullptr;
 
     BufferHeader* bh(ptr2BH(ptr));
 
@@ -151,8 +159,8 @@ gcache::Page::realloc (void* ptr, size_type size)
 
 void gcache::Page::print(std::ostream& os) const
 {
-    os << "page file: " << name() << ", size: " << size() << ", used: "
-       << used_;
+    os << "name: " << name() << ", size: " << size() << ", used: " << used()
+       << ", mapped: " << mapped_ << ", seqno_max: " << seqno_max();
 
     if (used_ > 0 && debug_ > 0)
     {
